// (c) Copyright 2013 Cloudera, Inc. All rights reserved.
define([
  'knockout',
  'underscore'
], function(ko, _) {
  var SearchResults = function(resultContainer, dataSource, modifyViewModel) {
    this.error = ko.observable();
    this.results = ko.observableArray();
    this.loading = ko.observable(false);
    this.dataSource = dataSource;

    var viewModel = {
      loading : this.loading,
      error : this.error,
      results : this.results
    };

    if (_.isFunction(modifyViewModel)) {
      modifyViewModel(viewModel);
    }

    ko.applyBindings(viewModel, resultContainer);
  };

  SearchResults.prototype = {
    getTail : function() {
      return this.length() ? this.results()[this.length() - 1] : null;
    },
    getHead : function() {
      return this.length() ? this.results()[0] : null;
    },
    length : function() {
      return this.results().length;
    },
    splice : function() {
      return this.results.splice
          .apply(this.results, $.makeArray(arguments));
    },
    prepend : function(newResult) {
      var result;
      if (_.isArray(newResult)) {
        this.results(this.length() ? newResult.concat(this.results()) : newResult);
        result = this.length();
      } else {
        result = this.results.unshift(newResult);
      }
      return result;
    },
    append : function(newResult) {
      var result;
      if (_.isArray(newResult)) {
        this.results(this.length() ? this.results().concat(newResult) : newResult);
        result = this.length();
      } else {
        result = this.results.push(newResult);
      }
      return result;
    },
    retryHandlerAfterTime: function(clear, handler, timeInMs) {
      var self = this;
      setTimeout(function() {
        self.dataSource(self, clear, handler);
      }, timeInMs);
    },
    makeLoadDataHandler: function(clear, callback) {
      var self = this;
      return function(err) {
        self.loading(false);

        if (err) {
          self.error(err);
        }

        if (callback && callback(err)) {
          var boundHandler = _.bind(self.makeLoadDataHandler, self);
          self.retryHandlerAfterTime(clear, boundHandler, 2000);
        }
      };
    },
    loadData : function(clear, callback) {
      if (_.isFunction(clear)) {
        callback = clear;
        clear = undefined;
      }

      this.error(undefined);
      this.loading(true);

      var handler = this.makeLoadDataHandler(clear, callback);
      this.dataSource(this, clear, handler);
    },
    clear : function() {
      this.results.splice(0);
    }
  };

  return SearchResults;
});